require "active_model"

module Opera::MobileStore
  class PaymentInfo

    include ActiveModel::Model
    include ActiveModel::Serialization

    include Opera::MobileStore::InspectableAttributes

    def type
      ref_type = self.class.name.demodulize.downcase
      ref_type == 'pay_pal' ? 'paypal' : ref_type
    end

    def self.build_from_nokogiri_node(node)
      if node.present?
        type = node.xpath("string(@type)").strip.downcase

        case type
        when "check"  then Check.build_from_nokogiri_node node
        when "wired"  then Wired.build_from_nokogiri_node node
        when "paypal" then PayPal.build_from_nokogiri_node node
        when "none"   then nil
        else raise "WTF?"
        end
      end
    end

    def self.deserialize(serializable_hash)
      attributes_hash = serializable_hash.inject({}) do |hsh, keyval|
        field_name, field_value = keyval

        case field_name
        when 'payment_info'
          field_value = PaymentInfo.deserialize field_value
        end

        hsh[field_name] = field_value
        hsh
      end

      self.new attributes_hash
    end

    class Check < PaymentInfo
      attr_accessor :name, :address

      def self.build_from_nokogiri_node(node)
        self.new(
          name:    node.xpath("string(payment_check_name)"),
          address: node.xpath("string(payment_check_address)")
        )
      end

      def attributes
        [:type, :name, :address].inject({}) do |hash, method|
          value = self.public_send method
          hash[method] = value unless value.nil?
          hash
        end
      end
    end

    class Wired < PaymentInfo
      attr_accessor :bank_account,
                    :bank_name,
                    :bank_address,
                    :bank_swiftbic,
                    :bank_iban,
                    :bank_routing_number,
                    :intermediary_bank_name,
                    :intermediary_bank_address,
                    :intermediary_bank_swiftbic,
                    :intermediary_bank_iban

      def self.build_from_nokogiri_node(node)
        data = [
          :bank_account, :bank_name, :bank_address, :bank_swiftbic, :bank_iban,
          :bank_routing_number, :intermediary_bank_name, :intermediary_bank_address,
          :intermediary_bank_swiftbic, :intermediary_bank_iban
        ].inject({}) do |hash, attribute_name|
          value = node.xpath("string(payment_wired_#{attribute_name})").strip
          hash[attribute_name] = value if value.present?
          hash
        end

        self.new data
      end

      def attributes
        [
          :type,
          :bank_account, :bank_name, :bank_address, :bank_swiftbic, :bank_iban,
          :bank_routing_number, :intermediary_bank_name, :intermediary_bank_address,
          :intermediary_bank_swiftbic, :intermediary_bank_iban
        ].inject({}) do |hash, method|
          value = self.public_send method
          hash[method] = value unless value.nil?
          hash
        end
      end
    end

    class PayPal < PaymentInfo
      attr_accessor :account
      def self.build_from_nokogiri_node(node)
        self.new(account: node.xpath("string(account)").strip)
      end

      def attributes
        { type: 'paypal', account: account }
      end
    end

  end
end
