require "faraday"
require "opera/mobile_store_sdk/faraday_middleware"

module Opera
  module MobileStoreSDK

    # Our version of Array... which will request each page upon iteration...
    # see http://stackoverflow.com/questions/1571349/can-the-array-be-reinvented-in-ruby
    class APIObjectList

      include Enumerable

      attr_reader :options

      def initialize(klass, options = {})
        @klass = klass
        @options = options.with_indifferent_access
      end

      def length
        response.body.count
      end

      def found_rows
        response.env[:found_rows]
      end
      alias_method :total_count, :found_rows

      def sdk_tms
        {
          api_calling: response.env[:opera_api_calling_tms],
          api_response_parsing: response.env[:opera_api_response_parsing_tms],
        }
      end

      def api_call_duration
        sdk_tms[:api_calling].real
      end

      def api_response_parsing_duration
        sdk_tms[:api_response_parsing].real
      end

      def api_response_datetime
        response.env[:opera_api_response_datetime]
      end

      def timestamp
        response.env[:timestamp]
      end

      def updated_at
        Time.at(page.timestamp).to_datetime
      end

      def [](n)
        response.body[n]
      end

      def each
        0.upto(length - 1) { |idx| yield self[idx] }
      end

      def to_a
        response.body
      end

      # Chainable Methods ------------------------------------------------------
      def where(given_options = {})

        given_options = given_options.with_indifferent_access

        ########################################################################
        # Validate (& filter out conflicting) options:

        # Product list API can use added_after/updated_after parameters:

        if given_options.include? :updated_after
          if given_options[:updated_after].is_a? String
            begin
              given_options[:updated_after] = DateTime.parse(given_options[:updated_after])
            rescue ArgumentError => e
              raise Opera::MobileStoreSDK::Errors::APIParamsError,
                "Parameter 'updated_after' is not a date (#{given_options[:updated_after]})"
            end
          elsif !given_options[:updated_after].respond_to? :to_date
            raise Opera::MobileStoreSDK::Errors::APIParamsError,
              "Parameter 'updated_after' is not a date (#{given_options[:updated_after]})"
          end
        end

        if given_options.key? :added_after
          if given_options[:added_after].is_a? String
            begin
              given_options[:added_after] = DateTime.parse(given_options[:added_after])
            rescue ArgumentError => e
              raise Opera::MobileStoreSDK::Errors::APIParamsError,
                "Parameter 'added_after' is not a date (#{given_options[:added_after]})"
            end
          elsif !given_options[:added_after].respond_to? :to_date
            raise Opera::MobileStoreSDK::Errors::APIParamsError,
              "Parameter 'added_after' is not a date (#{given_options[:added_after]})"
          end

          # Delete conflicting parameters:
          given_options.delete :updated_after
        end
        ########################################################################

        self.class.new @klass, @options.merge(given_options)
      end

      def page(given_page)
        where @options.merge(page: given_page)
      end

      def offset(given_offset)
        where @options.merge(offset: given_offset)
      end

      def per(given_item_count)
        where @options.merge(items: given_item_count)
      end

      def includes(included_fields = [])
        params = included_fields.map(&:to_s).map(&:downcase)
        .select { |x| %w(original_images billing eula adult subsites compatibility profit rating).include? x }
        .map { |x| x == "billing" ? "show_billing" : x }
        .inject({}) { |hash, param_name| hash[param_name] = "1"; hash }

        self.class.new @klass, @options.merge(params)
      end
      # ------------------------------------------------------------------------

      protected

        def response

          converted_options = @options.inject({}) do |hsh, keyval|
            key, val = keyval

            hsh[key.to_s] = case key.to_sym
              when :added_after, :updated_after then
                # byebug
                val.to_date.to_s
              else val.to_s
            end

            hsh
          end

          # puts "Options: #{@options.inspect}"
          # byebug

          @_response ||= Opera::MobileStoreSDK.connection.get do |request|
            request.url MobileStoreSDK.config.api_path, {
              action: action_param
            }.merge(converted_options)

            request.options.timeout = MobileStoreSDK.config.api_call_timeout
            request.options.open_timeout = MobileStoreSDK.config.api_call_open_timeout
          end
        end

        def action_param
          @_action_param ||= case @klass
            when "Opera::MobileStore::Developer" then "full_developers"
            when "Opera::MobileStore::Compatibility" then "compatibility"
            when "Opera::MobileStore::DeviceModel" then "models"
            when "Opera::MobileStore::DevicePlatform" then "platforms"
            when "Opera::MobileStore::DevicePlatformFamily" then "platform_families"
            else @klass.demodulize.underscore.pluralize
          end
        end

    end
  end
end
